@@ -10,10 +10,17 @@
     :license: BSD, see LICENSE for more details.
 """
 
-nodeenv_version = '0.11.0'
+nodeenv_version = '0.12.0'
 
 import sys
 import os
+import re
+try:
+    # python 2
+    from urllib2 import urlopen
+except ImportError:
+    # python 3
+    from urllib.request import urlopen
 import stat
 import logging
 import optparse
@@ -33,6 +40,7 @@ from pkg_resources import parse_version
 
 join = os.path.join
 abspath = os.path.abspath
+src_domain = "nodejs.org"
 
 # ---------------------------------------------------------
 # Utils
@@ -179,6 +187,11 @@ def parse_args(check=True):
         'The default is last stable version (`latest`). '
         'Use `system` to use system-wide node.')
 
+    parser.add_option(
+        '-i', '--iojs',
+        action='store_true', dest='io', default=False,
+        help='Use iojs instead of nodejs.')
+
     parser.add_option(
         '-j', '--jobs', dest='jobs', default=Config.jobs,
         help='Sets number of parallel commands at node.js compilation. '
@@ -327,36 +340,36 @@ def writefile(dest, content, overwrite=True, append=False):
     """
     Create file and write content in it
     """
+    content = content.encode('utf-8')
     if not os.path.exists(dest):
         logger.debug(' * Writing %s ... ', dest, extra=dict(continued=True))
-        f = open(dest, 'wb')
-        f.write(content.encode('utf-8'))
-        f.close()
+        with open(dest, 'wb') as f:
+            f.write(content)
         logger.debug('done.')
         return
     else:
-        f = open(dest, 'rb')
-        c = f.read()
-        f.close()
-        if c != content.encode('utf-8'):
-            if not overwrite:
-                logger.info(' * File %s exists with different content; '
-                            ' not overwriting', dest)
-                return
-            if append:
-                logger.info(' * Appending nodeenv settings to %s', dest)
-                f = open(dest, 'ab')
-                f.write(DISABLE_POMPT.encode('utf-8'))
-                f.write(content.encode('utf-8'))
-                f.write(ENABLE_PROMPT.encode('utf-8'))
-                f.close()
-                return
-            logger.info(' * Overwriting %s with new content', dest)
-            f = open(dest, 'wb')
-            f.write(content.encode('utf-8'))
-            f.close()
-        else:
+        with open(dest, 'rb') as f:
+            c = f.read()
+        if c == content:
             logger.debug(' * Content %s already in place', dest)
+            return
+
+        if not overwrite:
+            logger.info(' * File %s exists with different content; '
+                        ' not overwriting', dest)
+            return
+
+        if append:
+            logger.info(' * Appending data to %s', dest)
+            with open(dest, 'ab') as f:
+                f.write(DISABLE_POMPT.encode('utf-8'))
+                f.write(content)
+                f.write(ENABLE_PROMPT.encode('utf-8'))
+            return
+
+        logger.info(' * Overwriting %s with new content', dest)
+        with open(dest, 'wb') as f:
+            f.write(content)
 
 
 def callit(cmd, show_stdout=True, in_shell=False,
@@ -423,12 +436,12 @@ def callit(cmd, show_stdout=True, in_shell=False,
 
 
 def get_node_src_url(version, postfix=''):
-    node_name = 'node-v%s%s' % (version, postfix)
+    node_name = '%s-v%s%s' % (get_binary_prefix(), version, postfix)
     tar_name = '%s.tar.gz' % (node_name)
     if parse_version(version) > parse_version("0.5.0"):
-        node_url = 'http://nodejs.org/dist/v%s/%s' % (version, tar_name)
+        node_url = 'http://%s/dist/v%s/%s' % (src_domain, version, tar_name)
     else:
-        node_url = 'http://nodejs.org/dist/%s' % (tar_name)
+        node_url = 'http://%s/dist/%s' % (src_domain, tar_name)
     return node_url
 
 
@@ -480,7 +493,8 @@ def copy_node_from_prebuilt(env_dir, src_dir):
     Copy prebuilt binaries into environment
     """
     logger.info('.', extra=dict(continued=True))
-    callit(['cp', '-a', src_dir + '/node-v*/*', env_dir], True, env_dir)
+    prefix = get_binary_prefix()
+    callit(['cp', '-a', src_dir + '/%s-v*/*' % prefix, env_dir], True, env_dir)
     logger.info('.', extra=dict(continued=True))
 
 
@@ -539,16 +553,20 @@ def build_node_from_src(env_dir, src_dir, node_src_dir, opt):
     callit([make_cmd + ' install'], opt.verbose, True, node_src_dir, env)
 
 
+def get_binary_prefix():
+    return 'node' if src_domain == 'nodejs.org' else 'iojs'
+
 def install_node(env_dir, src_dir, opt):
     """
     Download source code for node.js, unpack it
     and install it in virtual environment.
     """
-    logger.info(' * Install node.js (%s' % opt.node,
+    prefix = get_binary_prefix()
+    logger.info(' * Install %s (%s' % (prefix, opt.node),
                 extra=dict(continued=True))
 
     node_url = get_node_src_url(opt.node, get_node_src_url_postfix(opt))
-    node_src_dir = join(src_dir, 'node-v%s' % (opt.node))
+    node_src_dir = join(src_dir, '%s-v%s' % (prefix, opt.node))
     env_dir = abspath(env_dir)
 
     # get src if not downloaded yet
@@ -694,9 +712,9 @@ def print_node_versions():
     Prints into stdout all available node.js versions
     """
     p = subprocess.Popen(
-        "curl -s http://nodejs.org/dist/ | "
-        "egrep -o '[0-9]+\.[0-9]+\.[0-9]+' | "
-        "sort -u -k 1,1n -k 2,2n -k 3,3n -t . ",
+        ("curl -s https://%s/dist/ | "
+         "egrep -o '[0-9]+\.[0-9]+\.[0-9]+' | "
+         "sort -u -k 1,1n -k 2,2n -k 3,3n -t . ") % (src_domain),
         shell=True, stdout=subprocess.PIPE)
     # out, err = p.communicate()
     pos = 0
@@ -717,14 +735,25 @@ def get_last_stable_node_version():
     """
     Return last stable node.js version
     """
-    p = subprocess.Popen(
-        "curl -s http://nodejs.org/dist/latest/ | "
-        "egrep -o 'node-v[0-9]+\.[0-9]+\.[0-9]+' | "
-        "sed -e 's/node-v//' | "
-        "sort -u -k 1,1n -k 2,2n -k 3,3n -t . | "
-        "tail -n1",
-        shell=True, stdout=subprocess.PIPE)
-    return p.stdout.read().decode("utf-8").replace("\n", "")
+    from lxml import etree
+
+    response = urlopen('http://%s/dist/latest/' % (src_domain))
+    html = etree.HTML(response.read())
+
+    links = []
+    pattern = re.compile(r'''%s-v([0-9]+)\.([0-9]+)\.([0-9]+)\.tar\.gz''' % (
+        get_binary_prefix()))
+
+    for a in html.xpath('//a'):
+        href = a.attrib.get('href', '')
+        match = pattern.match(href)
+        if match:
+            version = u'.'.join(match.groups())
+            major, minor, revision = map(int, match.groups())
+            links.append((version, major, minor, revision))
+            break
+
+    return links[-1][0]
 
 
 def get_env_dir(opt, args):
@@ -752,6 +781,11 @@ def main():
     Config._load(opt.config_file, opt.verbose)
 
     opt, args = parse_args()
+
+    if opt.io:
+        global src_domain
+        src_domain = "iojs.org"
+
     if not opt.node or opt.node.lower() == "latest":
         opt.node = get_last_stable_node_version()
 
@@ -782,6 +816,7 @@ unset NODE_VIRTUAL_ENV_DISABLE_PROMPT
 SHIM = """#!/usr/bin/env bash
 export NODE_PATH=__NODE_VIRTUAL_ENV__/lib/node_modules
 export NPM_CONFIG_PREFIX=__NODE_VIRTUAL_ENV__
+export npm_config_prefix=__NODE_VIRTUAL_ENV__
 exec __SHIM_NODE__ "$@"
 """
 
@@ -802,8 +837,11 @@ deactivate_node () {
         unset _OLD_NODE_PATH
 
         NPM_CONFIG_PREFIX="$_OLD_NPM_CONFIG_PREFIX"
+        npm_config_prefix="$_OLD_npm_config_prefix"
         export NPM_CONFIG_PREFIX
+        export npm_config_prefix
         unset _OLD_NPM_CONFIG_PREFIX
+        unset _OLD_npm_config_prefix
     fi
 
     # This should detect bash and zsh, which have a hash command that must
@@ -879,8 +917,11 @@ NODE_PATH="$NODE_VIRTUAL_ENV/__MOD_NAME__"
 export NODE_PATH
 
 _OLD_NPM_CONFIG_PREFIX="$NPM_CONFIG_PREFIX"
+_OLD_npm_config_prefix="$npm_config_prefix"
 NPM_CONFIG_PREFIX="$NODE_VIRTUAL_ENV"
+npm_config_prefix="$NODE_VIRTUAL_ENV"
 export NPM_CONFIG_PREFIX
+export npm_config_prefix
 
 if [ -z "$NODE_VIRTUAL_ENV_DISABLE_PROMPT" ] ; then
     _OLD_NODE_VIRTUAL_PS1="$PS1"
